/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor.fritzmeijer;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.bean.ColumnPositionMappingStrategy;
import au.com.bytecode.opencsv.bean.CsvToBean;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.io.StringReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorData;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorProcessor;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.filesystems.FileObject;
import org.openide.util.NbBundle;

/**
 * Fritzmeijer File processor Processes Fritzmeijer sensor files
 *
 * @author Gerben Feenstra
 */
public class FritzmeijerFileProcessor extends SensorProcessor<FritzmeijerSensorData> {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_FAILED_TO_CREATE_DATE_FROM_FILENAME = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_create_date_from_filename");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_start");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(FritzmeijerFileProcessor.class, "sensor.fritzmeijer.fileprocessor.log.displayname");
    // Other variable declarations
    private final String delimiter = FritzmeijerFileDescription.getDelimiter();
    private final ProgressHandle progressHandle;
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("hh:mm:ss");
    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("DD.MM.yyyy");
    //Tested. mem consumtion diference between 10 and 20 thuosand is 3MB. 20 vs 30 is 50 MB
    private static final int BULK_COMMIT_AMOUNT = 1000;
    protected FritzmeijerDataObject dataObject;
    PropertyEditor old;
    private static final String[] COLUMS = new String[]{"latitude", "longitude", "ddateStr", "utcTimeStr", "course", "latitudeOriginal", "longitudeOriginal", "altitude", "speed", "fixType", "irmi", "tcari", "osavi", "tcOs", "wdvi", "ndre"};
    private CsvToBean csv;
    private ColumnPositionMappingStrategy strat;
    private static final AppLogger LOG = AppLogFactory.getLogger(FritzmeijerFileProcessor.class);

    public FritzmeijerFileProcessor(FritzmeijerDataObject toProcess) {
        // assert dataObject != null;
        LOG.entering("FritzmeijerFileProcessor", "constructor");
        this.dataObject = toProcess;
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_description", dataObject.getName()));
        old = PropertyEditorManager.findEditor(Date.class);
        strat = new ColumnPositionMappingStrategy();
        strat.setType(FritzmeijerSensorData.class);
        strat.setColumnMapping(COLUMS);
        csv = new CsvToBean();
    }

    /**
     * Parses payload to FritzmeijerSensorData
     *
     * @param payload Fritzmeijer sensor line
     * @param dataFile
     * @return FritzmeijerSensorData object
     * @throws ParseException
     * @throws FritzmeijerParseException
     */
    public FritzmeijerSensorData parse(String payload) throws FritzmeijerParseException {
        try {
            // Use delimiter defined in FritzmeijerFileDescription
            CSVReader csvReader = new CSVReader(new StringReader(payload), delimiter.charAt(0));
            List<FritzmeijerSensorData> list = csv.parse(strat, csvReader);
            return (FritzmeijerSensorData) list.get(0);
        } catch (RuntimeException ex) {
            LOG.error(ex.getMessage());
            throw new FritzmeijerParseException(ex.getCause());
        }
    }

    public boolean isHeader(String line) {
        return line.startsWith("$");
    }

    @Override
    public List<SensorData> process() {
        List<SensorData> sensorData = new ArrayList<SensorData>();

        try {
            int success = 0;
            int progress = 0;
            int fail = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), dataObject.getName());
            FileObject fileObject = dataObject.getPrimaryFile();

            try {
                this.progressHandle.start(Long.valueOf(fileObject.asLines().size()).intValue());

                for (String line : fileObject.asLines()) {
                    progressHandle.progress(++progress);
                    if (!isHeader(line)) {
                        try {
                            sensorData.add(parse(line));
                            success++;
                        } catch (Exception ex) {
                            fail++;
                        }
                    }
                }
                LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);
                if (fail > 0) {
                    LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
                }
            } finally {
                progressHandle.finish();
            }
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, dataObject.getName());
            LOG.trace(ex);
        }
        return sensorData;
    }

    /**
     * Method which will create a SensorMetadata object from the preferences:
     * FritzmeijerPreference.SENSOR_DATA_FILE_FORMAT
     * FritzmeijerPreference.CROPCIRCLE_SENSOR_HEIGTH
     * FritzmeijerPreference.NUMBER_OF_CROPCIRCLE_SENSORS
     * FritzmeijerPreference.CROPCIRCLE_SENSOR_SPREAD
     * FritzmeijerPreference.SENSOR_TYPE FritzmeijerPreference.SPECTRAL_CHANNELS
     *
     * @return created SensorMetadata object
     */
    @Override
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = FritzmeijerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
        int sensorHeigth = Integer.parseInt(FritzmeijerPreference.FRITZMEIJER_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(FritzmeijerPreference.NUMBER_OF_FRITZMEIJER_SENSORS.getValue());
        String FritzmeijerSensorSpread = FritzmeijerPreference.FRITZMEIJER_SENSOR_SPREAD.getValue();
        String sensorType = FritzmeijerPreference.SENSOR_TYPE.getValue();
        String spectralChannels = FritzmeijerPreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, FritzmeijerSensorSpread, sensorType, spectralChannels);
    }

    @Override
    protected String getFileDateFormat() {
        return FritzmeijerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }
}
